@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -54,8 +54,10 @@ int main(int argc, char **argv)
     xformWidget.setStyle(arthurStyle);
 
     QList<QWidget *> widgets = qFindChildren<QWidget *>(&xformWidget);
-    foreach (QWidget *w, widgets)
+    foreach (QWidget *w, widgets) {
         w->setStyle(arthurStyle);
+        w->setAttribute(Qt::WA_AcceptTouchEvents);
+    }
 
     xformWidget.show();
 
