@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -48,22 +48,24 @@ int main(int argc, char **argv)
 
     QApplication app(argc, argv);
 
-    bool smallScreen = false;
-    for (int i=0; i<argc; i++)
-        if (QString(argv[i]) == "-small-screen")
-            smallScreen = true;
+    bool smallScreen = QApplication::arguments().contains("-small-screen");
 
     PathStrokeWidget pathStrokeWidget(smallScreen);
     QStyle *arthurStyle = new ArthurStyle();
     pathStrokeWidget.setStyle(arthurStyle);
     QList<QWidget *> widgets = qFindChildren<QWidget *>(&pathStrokeWidget);
-    foreach (QWidget *w, widgets)
+    foreach (QWidget *w, widgets) {
         w->setStyle(arthurStyle);
+        w->setAttribute(Qt::WA_AcceptTouchEvents);
+    }
 
     if (smallScreen)
         pathStrokeWidget.showFullScreen();
     else
         pathStrokeWidget.show();
 
+#ifdef QT_KEYPAD_NAVIGATION
+    QApplication::setNavigationMode(Qt::NavigationModeCursorAuto);
+#endif
     return app.exec();
 }
