@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -79,14 +79,20 @@ void ExampleContent::animationStopped(int id)
 QString ExampleContent::loadDescription()
 {
     QByteArray ba = MenuManager::instance()->getHtml(this->name);
+    QString errorMsg;
+    int errorLine, errorColumn;
 
     QDomDocument exampleDoc;
-    exampleDoc.setContent(ba, false);
+    if (!exampleDoc.setContent(ba, false, &errorMsg, &errorLine, &errorColumn)) {
+        qDebug() << errorMsg << errorLine << errorColumn;
+    }
 
     QDomNodeList paragraphs = exampleDoc.elementsByTagName("p");
     if (paragraphs.length() < 1 && Colors::verbose)
-        qDebug() << "- ExampleContent::loadDescription(): Could not load description:" << MenuManager::instance()->info[this->name]["docfile"];
-    QString description = Colors::contentColor + QLatin1String("Could not load description. Ensure that the documentation for Qt is built.");
+        qDebug() << "- ExampleContent::loadDescription(): Could not load description:"
+                 << MenuManager::instance()->info[this->name]["docfile"];
+    QString description = Colors::contentColor + QLatin1String(""); 
+    //QLatin1String("Could not load description. Ensure that the documentation for Qt is built."); // QTBUG-12522: If there is no description why show an error to the user when qDebug above communications the issue (if it is indeed an issue at all) when demos are built?
     for (int p = 0; p < int(paragraphs.length()); ++p) {
         description = this->extractTextFromParagraph(paragraphs.item(p));
         if (this->isSummary(description)) {
@@ -99,7 +105,8 @@ QString ExampleContent::loadDescription()
 bool ExampleContent::isSummary(const QString &text)
 {
     return (!text.contains("[") &&
-        text.indexOf(QRegExp(QString("(In )?((The|This) )?(%1 )?.*(tutorial|example|demo|application)").arg(this->name), Qt::CaseInsensitive)) != -1);
+        text.indexOf(QRegExp(QString("(In )?((The|This) )?(%1 )?.*(tutorial|example|demo|application)").arg(this->name),
+                             Qt::CaseInsensitive)) != -1);
 }
 
 QString ExampleContent::extractTextFromParagraph(const QDomNode &parentNode)
