@@ -1,23 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** All rights reserved.
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** This file is part of the example classes of the Qt Toolkit.
+** This file is part of the examples of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** $QT_BEGIN_LICENSE:BSD$
+** You may use this file under the terms of the BSD license as follows:
 **
-** If you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** "Redistribution and use in source and binary forms, with or without
+** modification, are permitted provided that the following conditions are
+** met:
+**   * Redistributions of source code must retain the above copyright
+**     notice, this list of conditions and the following disclaimer.
+**   * Redistributions in binary form must reproduce the above copyright
+**     notice, this list of conditions and the following disclaimer in
+**     the documentation and/or other materials provided with the
+**     distribution.
+**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
+**     the names of its contributors may be used to endorse or promote
+**     products derived from this software without specific prior written
+**     permission.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -31,10 +48,7 @@
 #include "ratecontroller.h"
 
 #include <QtCore>
-
-extern "C" {
-#include "3rdparty/sha1.h"
-}
+#include <QNetworkInterface>
 
 // These constants could also be configurable by the user.
 static const int ServerMinPort = 6881;
@@ -210,20 +224,20 @@ TorrentClient::TorrentClient(QObject *parent)
     : QObject(parent), d(new TorrentClientPrivate(this))
 {
     // Connect the file manager
-    connect(&d->fileManager, SIGNAL(dataRead(int, int, int, const QByteArray &)),
-            this, SLOT(sendToPeer(int, int, int, const QByteArray &)));
+    connect(&d->fileManager, SIGNAL(dataRead(int,int,int,QByteArray)),
+            this, SLOT(sendToPeer(int,int,int,QByteArray)));
     connect(&d->fileManager, SIGNAL(verificationProgress(int)),
             this, SLOT(updateProgress(int)));
     connect(&d->fileManager, SIGNAL(verificationDone()),
             this, SLOT(fullVerificationDone()));
-    connect(&d->fileManager, SIGNAL(pieceVerified(int, bool)),
-            this, SLOT(pieceVerified(int, bool)));
+    connect(&d->fileManager, SIGNAL(pieceVerified(int,bool)),
+            this, SLOT(pieceVerified(int,bool)));
     connect(&d->fileManager, SIGNAL(error()),
             this, SLOT(handleFileError()));
 
     // Connect the tracker client
-    connect(&d->trackerClient, SIGNAL(peerListUpdated(const QList<TorrentPeer> &)),
-            this, SLOT(addToPeerList(const QList<TorrentPeer> &)));
+    connect(&d->trackerClient, SIGNAL(peerListUpdated(QList<TorrentPeer>)),
+            this, SLOT(addToPeerList(QList<TorrentPeer>)));
     connect(&d->trackerClient, SIGNAL(stopped()),
             this, SIGNAL(stopped()));
 }
@@ -254,26 +268,7 @@ bool TorrentClient::setTorrent(const QByteArray &torrentData)
 
     // Calculate SHA1 hash of the "info" section in the torrent
     QByteArray infoValue = d->metaInfo.infoValue();
-    SHA1Context sha;
-    SHA1Reset(&sha);
-    SHA1Input(&sha, (const unsigned char *)infoValue.constData(), infoValue.size());
-    SHA1Result(&sha);
-    unsigned char *digest = (unsigned char *)sha.Message_Digest;
-    d->infoHash.resize(20);
-
-    for (int i = 0; i < 5; ++i) {
-#if Q_BYTE_ORDER == Q_BIG_ENDIAN
-        d->infoHash[i * 4 + 3] = digest[i * 4 + 3];
-        d->infoHash[i * 4 + 2] = digest[i * 4 + 2];
-        d->infoHash[i * 4 + 1] = digest[i * 4 + 1];
-        d->infoHash[i * 4 + 0] = digest[i * 4 + 0];
-#else
-        d->infoHash[i * 4 + 0] = digest[i * 4 + 3];
-        d->infoHash[i * 4 + 1] = digest[i * 4 + 2];
-        d->infoHash[i * 4 + 2] = digest[i * 4 + 1];
-        d->infoHash[i * 4 + 3] = digest[i * 4 + 0];
-#endif
-    }
+    d->infoHash = QCryptographicHash::hash(infoValue, QCryptographicHash::Sha1);
 
     return true;
 }
@@ -458,8 +453,11 @@ void TorrentClient::stop()
     }
 
     // Abort all existing connections
-    foreach (PeerWireClient *client, d->connections)
+    foreach (PeerWireClient *client, d->connections) {
+        RateController::instance()->removeSocket(client);
+        ConnectionManager::instance()->removeConnection(client);
         client->abort();
+    }
     d->connections.clear();
 
     // Perhaps stop the tracker
@@ -586,8 +584,6 @@ void TorrentClient::fullVerificationDone()
     d->setState(d->completedPieces.count(true) == d->pieceCount ? Seeding : Searching);
 
     // Start the tracker client
-    d->trackerClient.setUploadCount(d->uploadedBytes);
-    d->trackerClient.setDownloadCount(d->downloadedBytes);
     d->trackerClient.start(d->metaInfo);
 }
 
@@ -648,7 +644,7 @@ void TorrentClient::pieceVerified(int pieceIndex, bool ok)
     if (completed == d->pieceCount) {
         if (d->state != Seeding) {
             d->setState(Seeding);
-            d->trackerClient.start(d->metaInfo);
+            d->trackerClient.startSeeding();
         }
     } else {
         if (completed == 1)
@@ -685,7 +681,7 @@ void TorrentClient::connectToPeers()
 
     // Start as many connections as we can
     while (!weighedPeers.isEmpty() && ConnectionManager::instance()->canAddConnection()
-           && (rand() % (ConnectionManager::instance()->maxConnections() / 2))) {
+           && (qrand() % (ConnectionManager::instance()->maxConnections() / 2))) {
         PeerWireClient *client = new PeerWireClient(ConnectionManager::instance()->clientId(), this);
         RateController::instance()->addSocket(client);
         ConnectionManager::instance()->addConnection(client);
@@ -694,7 +690,7 @@ void TorrentClient::connectToPeers()
         d->connections << client;
 
         // Pick a random peer from the list of weighed peers.
-        TorrentPeer *peer = weighedPeers.takeAt(rand() % weighedPeers.size());
+        TorrentPeer *peer = weighedPeers.takeAt(qrand() % weighedPeers.size());
         weighedPeers.removeAll(peer);
         peer->connectStart = QDateTime::currentDateTime().toTime_t();
         peer->lastVisited = peer->connectStart;
@@ -839,12 +835,12 @@ void TorrentClient::initializeConnection(PeerWireClient *client)
             this, SLOT(removeClient()));
     connect(client, SIGNAL(error(QAbstractSocket::SocketError)),
             this, SLOT(removeClient()));
-    connect(client, SIGNAL(piecesAvailable(const QBitArray &)),
-            this, SLOT(peerPiecesAvailable(const QBitArray &)));
-    connect(client, SIGNAL(blockRequested(int, int, int)),
-            this, SLOT(peerRequestsBlock(int, int, int)));
-    connect(client, SIGNAL(blockReceived(int, int, const QByteArray &)),
-            this, SLOT(blockReceived(int, int, const QByteArray &)));
+    connect(client, SIGNAL(piecesAvailable(QBitArray)),
+            this, SLOT(peerPiecesAvailable(QBitArray)));
+    connect(client, SIGNAL(blockRequested(int,int,int)),
+            this, SLOT(peerRequestsBlock(int,int,int)));
+    connect(client, SIGNAL(blockReceived(int,int,QByteArray)),
+            this, SLOT(blockReceived(int,int,QByteArray)));
     connect(client, SIGNAL(choked()),
             this, SLOT(peerChoked()));
     connect(client, SIGNAL(unchoked()),
@@ -1107,7 +1103,7 @@ void TorrentClient::scheduleUploads()
         }
 
         if ((client->peerWireState() & PeerWireClient::ChokingPeer) == 0) {
-            if ((rand() % 10) == 0) 
+            if ((qrand() % 10) == 0) 
                 client->abort();
             else
                 client->chokePeer();
@@ -1121,7 +1117,7 @@ void TorrentClient::scheduleUploads()
     // random peer to allow it to compete for a position among the
     // downloaders.  (This is known as an "optimistic unchoke".)
     if (!allClients.isEmpty()) {
-        PeerWireClient *client = allClients[rand() % allClients.size()];
+        PeerWireClient *client = allClients[qrand() % allClients.size()];
         if (client->peerWireState() & PeerWireClient::ChokingPeer)
             client->unchokePeer();
     }
@@ -1182,7 +1178,7 @@ void TorrentClient::schedulePieceForClient(PeerWireClient *client)
         piece = d->payloads.value(client);
         if (!piece) {
             QList<TorrentPiece *> values = d->pendingPieces.values();
-            piece = values.value(rand() % values.size());
+            piece = values.value(qrand() % values.size());
             piece->inProgress = true;
             d->payloads.insert(client, piece);
         }
@@ -1239,16 +1235,16 @@ void TorrentClient::schedulePieceForClient(PeerWireClient *client)
             ++it;
         }
         if (!partialPieces.isEmpty())
-            piece = partialPieces.value(rand() % partialPieces.size());
+            piece = partialPieces.value(qrand() % partialPieces.size());
 
         if (!piece) {
             // Pick a random piece 3 out of 4 times; otherwise, pick either
             // one of the most common or the least common pieces available,
             // depending on the state we're in.
             int pieceIndex = 0;
-            if (d->state == WarmingUp || (rand() & 4) == 0) {
-                int *occurrances = new int[d->pieceCount];
-                memset(occurrances, 0, d->pieceCount * sizeof(int));
+            if (d->state == WarmingUp || (qrand() & 4) == 0) {
+                int *occurrences = new int[d->pieceCount];
+                memset(occurrences, 0, d->pieceCount * sizeof(int));
                 
                 // Count how many of each piece are available.
                 foreach (PeerWireClient *peer, d->connections) {
@@ -1256,38 +1252,38 @@ void TorrentClient::schedulePieceForClient(PeerWireClient *client)
                     int peerPiecesSize = peerPieces.size();
                     for (int i = 0; i < peerPiecesSize; ++i) {
                         if (peerPieces.testBit(i))
-                            ++occurrances[i];
+                            ++occurrences[i];
                     }
                 }
 
                 // Find the rarest or most common pieces.
-                int numOccurrances = d->state == WarmingUp ? 0 : 99999;
+                int numOccurrences = d->state == WarmingUp ? 0 : 99999;
                 QList<int> piecesReadyForDownload;
                 for (int i = 0; i < d->pieceCount; ++i) {
                     if (d->state == WarmingUp) {
                         // Add common pieces
-                        if (occurrances[i] >= numOccurrances
+                        if (occurrences[i] >= numOccurrences
                             && incompletePiecesAvailableToClient.testBit(i)) {
-                            if (occurrances[i] > numOccurrances)
+                            if (occurrences[i] > numOccurrences)
                                 piecesReadyForDownload.clear();
                             piecesReadyForDownload.append(i);
-                            numOccurrances = occurrances[i];
+                            numOccurrences = occurrences[i];
                         }
                     } else {
                         // Add rare pieces
-                        if (occurrances[i] <= numOccurrances
+                        if (occurrences[i] <= numOccurrences
                             && incompletePiecesAvailableToClient.testBit(i)) {
-                            if (occurrances[i] < numOccurrances)
+                            if (occurrences[i] < numOccurrences)
                                 piecesReadyForDownload.clear();
                             piecesReadyForDownload.append(i);
-                            numOccurrances = occurrances[i];
+                            numOccurrences = occurrences[i];
                         }
                     }
                 }
 
                 // Select one piece randomly
-                pieceIndex = piecesReadyForDownload.at(rand() % piecesReadyForDownload.size());
-                delete [] occurrances;
+                pieceIndex = piecesReadyForDownload.at(qrand() % piecesReadyForDownload.size());
+                delete [] occurrences;
             } else {
                 // Make up a list of available piece indices, and pick
                 // a random one.
@@ -1297,7 +1293,7 @@ void TorrentClient::schedulePieceForClient(PeerWireClient *client)
                     if (incompletePiecesAvailableToClient.testBit(i))
                         values << i;
                 }
-                pieceIndex = values.at(rand() % values.size());
+                pieceIndex = values.at(qrand() % values.size());
             }
 
             // Create a new TorrentPiece and fill in all initial
@@ -1389,8 +1385,8 @@ int TorrentClient::requestBlocks(PeerWireClient *client, TorrentPiece *piece, in
         // speedup comes from an increased chance of receiving
         // different blocks from the different peers.
         for (int i = 0; i < bits.size(); ++i) {
-            int a = rand() % bits.size();
-            int b = rand() % bits.size();
+            int a = qrand() % bits.size();
+            int b = qrand() % bits.size();
             int tmp = bits[a];
             bits[a] = bits[b];
             bits[b] = tmp;
@@ -1443,10 +1439,14 @@ void TorrentClient::peerUnchoked()
 void TorrentClient::addToPeerList(const QList<TorrentPeer> &peerList)
 {
     // Add peers we don't already know of to our list of peers.
+    QList<QHostAddress> addresses =  QNetworkInterface::allAddresses();
     foreach (TorrentPeer peer, peerList) {
-        if (peer.address == TorrentServer::instance()->serverAddress()
-            && peer.port == TorrentServer::instance()->serverPort())
+        if (addresses.contains(peer.address)
+            && peer.port == TorrentServer::instance()->serverPort()) {
+            // Skip our own server.
             continue;
+        }
+		
         bool known = false;
         foreach (TorrentPeer *knownPeer, d->peers) {
             if (knownPeer->port == peer.port
